@@ -3,12 +3,13 @@ mod server;
 pub mod subnet;
 
 use clock_steering::unix::UnixClock;
-#[cfg(feature = "unstable_ntpv5")]
-use ntp_proto::NtpVersion;
-use ntp_proto::{AlgorithmConfig, SourceDefaultsConfig, SynchronizationConfig};
+use ntp_proto::{
+    AlgorithmConfig, NtpVersion, ProtocolVersion, SourceConfig, SynchronizationConfig,
+};
 pub use ntp_source::*;
 use serde::{Deserialize, Deserializer};
 pub use server::*;
+use std::io;
 use std::{
     fmt::Display,
     io::ErrorKind,
@@ -18,7 +19,6 @@ use std::{
     str::FromStr,
 };
 use timestamped_socket::interface::InterfaceName;
-use tokio::{fs::read_to_string, io};
 use tracing::{info, warn};
 
 use super::{clock::NtpClockWrapper, tracing::LogLevel};
@@ -116,7 +116,7 @@ impl CliArg {
                                 // short version of --help has no arguments
                                 processed.push(CliArg::Flag(flag));
                             } else {
-                                Err(format!("'-{}' expects an argument", char))?;
+                                Err(format!("'-{char}' expects an argument"))?;
                             }
                             break;
                         } else {
@@ -362,7 +362,7 @@ pub struct Config {
     #[serde(default)]
     pub synchronization: DaemonSynchronizationConfig,
     #[serde(default)]
-    pub source_defaults: SourceDefaultsConfig,
+    pub source_defaults: SourceConfig,
     #[serde(default)]
     pub observability: ObservabilityConfig,
     #[serde(default)]
@@ -373,7 +373,7 @@ pub struct Config {
 }
 
 impl Config {
-    async fn from_file(file: impl AsRef<Path>) -> Result<Config, ConfigError> {
+    fn from_file(file: impl AsRef<Path>) -> Result<Config, ConfigError> {
         let meta = std::fs::metadata(&file)?;
         let perm = meta.permissions();
 
@@ -381,23 +381,23 @@ impl Config {
             warn!("Unrestricted config file permissions: Others can write.");
         }
 
-        let contents = read_to_string(file).await?;
+        let contents = std::fs::read_to_string(file)?;
         Ok(toml::de::from_str(&contents)?)
     }
 
-    async fn from_first_file(file: Option<impl AsRef<Path>>) -> Result<Config, ConfigError> {
+    fn from_first_file(file: Option<impl AsRef<Path>>) -> Result<Config, ConfigError> {
         // if an explicit file is given, always use that one
         if let Some(f) = file {
             let path: &Path = f.as_ref();
             info!(?path, "using config file");
-            return Config::from_file(f).await;
+            return Config::from_file(f);
         }
 
         // for the global file we also ignore it when there are permission errors
         let global_path = Path::new("/etc/ntpd-rs/ntp.toml");
         if global_path.exists() {
             info!("using config file at default location `{:?}`", global_path);
-            match Config::from_file(global_path).await {
+            match Config::from_file(global_path) {
                 Err(ConfigError::Io(e)) if e.kind() == ErrorKind::PermissionDenied => {
                     warn!("permission denied on global config file! using default config ...");
                 }
@@ -410,12 +410,12 @@ impl Config {
         Ok(Config::default())
     }
 
-    pub async fn from_args(
+    pub fn from_args(
         file: Option<impl AsRef<Path>>,
         sources: Vec<NtpSourceConfig>,
         servers: Vec<ServerConfig>,
     ) -> Result<Config, ConfigError> {
-        let mut config = Config::from_first_file(file.as_ref()).await?;
+        let mut config = Config::from_first_file(file.as_ref())?;
 
         if !sources.is_empty() {
             if !config.sources.is_empty() {
@@ -441,10 +441,12 @@ impl Config {
             match source {
                 NtpSourceConfig::Standard(_) => count += 1,
                 NtpSourceConfig::Nts(_) => count += 1,
-                NtpSourceConfig::Pool(config) => count += config.count,
+                NtpSourceConfig::Pool(config) => count += config.first.count,
                 #[cfg(feature = "unstable_nts-pool")]
-                NtpSourceConfig::NtsPool(config) => count += config.count,
+                NtpSourceConfig::NtsPool(config) => count += config.first.count,
                 NtpSourceConfig::Sock(_) => count += 1,
+                #[cfg(feature = "pps")]
+                NtpSourceConfig::Pps(_) => {} // PPS sources don't count
             }
         }
         count
@@ -474,19 +476,54 @@ impl Config {
             ok = false;
         }
 
-        #[cfg(feature = "unstable_ntpv5")]
         if self.sources.iter().any(|config| match config {
             NtpSourceConfig::Sock(_) => false,
-            NtpSourceConfig::Standard(config) => matches!(config.ntp_version, Some(NtpVersion::V5)),
-            NtpSourceConfig::Nts(config) => matches!(config.ntp_version, Some(NtpVersion::V5)),
-            NtpSourceConfig::Pool(config) => matches!(config.ntp_version, Some(NtpVersion::V5)),
+            NtpSourceConfig::Pps(_) => false,
+            NtpSourceConfig::Standard(config) => {
+                matches!(config.first.ntp_version, ProtocolVersion::V5)
+            }
+            NtpSourceConfig::Nts(config) => {
+                matches!(config.first.ntp_version, ProtocolVersion::V5)
+            }
+            NtpSourceConfig::Pool(config) => {
+                matches!(config.first.ntp_version, ProtocolVersion::V5)
+            }
             #[cfg(feature = "unstable_nts-pool")]
-            NtpSourceConfig::NtsPool(config) => matches!(config.ntp_version, Some(NtpVersion::V5)),
+            NtpSourceConfig::NtsPool(config) => {
+                matches!(config.first.ntp_version, ProtocolVersion::V5)
+            }
         }) {
             warn!("Forcing a source into NTPv5, which is still a draft. There is no guarantee that the server will remain compatible with this or future versions of ntpd-rs.");
             ok = false;
         }
 
+        // Check that the NTS configuration is consistent with the NTP configuration
+        for ke_server in self
+            .nts_ke
+            .iter()
+            .filter(|ke_server| ke_server.ntp_server.is_none())
+        {
+            if ke_server.accept_ntp_versions.contains(&NtpVersion::V4)
+                && !self.servers.iter().any(|server| {
+                    server.listen.port() == ke_server.ntp_port.unwrap_or(123)
+                        && server.accept_ntp_versions.contains(&NtpVersion::V4)
+                })
+            {
+                warn!("Configured NTS for NTPv4 on port {}, but have no server listening on that port for NTPv4 traffic. If this is for an external ntp server, consider configuring a value for `ntp-server`.", ke_server.ntp_port.unwrap_or(123));
+                ok = false;
+            }
+
+            if ke_server.accept_ntp_versions.contains(&NtpVersion::V5)
+                && !self.servers.iter().any(|server| {
+                    server.listen.port() == ke_server.ntp_port.unwrap_or(123)
+                        && server.accept_ntp_versions.contains(&NtpVersion::V5)
+                })
+            {
+                warn!("Configured NTS for NTPv5 on port {}, but have no server listening on that port for NTPv5 traffic. If this is for an external ntp server, consider configuring a value for `ntp-server`.", ke_server.ntp_port.unwrap_or(123));
+                ok = false;
+            }
+        }
+
         ok
     }
 }
@@ -522,7 +559,7 @@ impl From<toml::de::Error> for ConfigError {
 
 #[cfg(test)]
 mod tests {
-    use ntp_proto::{NtpDuration, StepThreshold};
+    use ntp_proto::{NtpDuration, ProtocolVersion, StepThreshold};
 
     use super::*;
 
@@ -532,10 +569,12 @@ mod tests {
             toml::from_str("[[source]]\nmode = \"server\"\naddress = \"example.com\"").unwrap();
         assert_eq!(
             config.sources,
-            vec![NtpSourceConfig::Standard(StandardSource {
-                address: NormalizedAddress::new_unchecked("example.com", 123).into(),
-                #[cfg(feature = "unstable_ntpv5")]
-                ntp_version: None,
+            vec![NtpSourceConfig::Standard(FlattenedPair {
+                first: StandardSource {
+                    address: NormalizedAddress::new_unchecked("example.com", 123).into(),
+                    ntp_version: ProtocolVersion::V4,
+                },
+                second: Default::default()
             })]
         );
         assert!(config.observability.log_level.is_none());
@@ -547,10 +586,12 @@ mod tests {
         assert_eq!(config.observability.log_level, Some(LogLevel::Info));
         assert_eq!(
             config.sources,
-            vec![NtpSourceConfig::Standard(StandardSource {
-                address: NormalizedAddress::new_unchecked("example.com", 123).into(),
-                #[cfg(feature = "unstable_ntpv5")]
-                ntp_version: None,
+            vec![NtpSourceConfig::Standard(FlattenedPair {
+                first: StandardSource {
+                    address: NormalizedAddress::new_unchecked("example.com", 123).into(),
+                    ntp_version: ProtocolVersion::V4,
+                },
+                second: Default::default()
             })]
         );
 
@@ -560,10 +601,12 @@ mod tests {
             .unwrap();
         assert_eq!(
             config.sources,
-            vec![NtpSourceConfig::Standard(StandardSource {
-                address: NormalizedAddress::new_unchecked("example.com", 123).into(),
-                #[cfg(feature = "unstable_ntpv5")]
-                ntp_version: None,
+            vec![NtpSourceConfig::Standard(FlattenedPair {
+                first: StandardSource {
+                    address: NormalizedAddress::new_unchecked("example.com", 123).into(),
+                    ntp_version: ProtocolVersion::V4,
+                },
+                second: Default::default()
             })]
         );
         assert_eq!(
@@ -589,10 +632,12 @@ mod tests {
             .unwrap();
         assert_eq!(
             config.sources,
-            vec![NtpSourceConfig::Standard(StandardSource {
-                address: NormalizedAddress::new_unchecked("example.com", 123).into(),
-                #[cfg(feature = "unstable_ntpv5")]
-                ntp_version: None,
+            vec![NtpSourceConfig::Standard(FlattenedPair {
+                first: StandardSource {
+                    address: NormalizedAddress::new_unchecked("example.com", 123).into(),
+                    ntp_version: ProtocolVersion::V4,
+                },
+                second: Default::default()
             })]
         );
         assert!(config
@@ -633,10 +678,12 @@ mod tests {
 
         assert_eq!(
             config.sources,
-            vec![NtpSourceConfig::Standard(StandardSource {
-                address: NormalizedAddress::new_unchecked("example.com", 123).into(),
-                #[cfg(feature = "unstable_ntpv5")]
-                ntp_version: None,
+            vec![NtpSourceConfig::Standard(FlattenedPair {
+                first: StandardSource {
+                    address: NormalizedAddress::new_unchecked("example.com", 123).into(),
+                    ntp_version: ProtocolVersion::V4,
+                },
+                second: Default::default()
             })]
         );
 
@@ -645,6 +692,22 @@ mod tests {
         assert_eq!(poll_interval_limits.max.as_log(), 9);
 
         assert_eq!(config.source_defaults.initial_poll_interval.as_log(), 5);
+
+        let config: Config = toml::from_str(
+            "[[source]]\nmode = \"server\"\naddress = \"example.com\"\nntp-version = \"auto\"",
+        )
+        .unwrap();
+        assert_eq!(
+            config.sources,
+            vec![NtpSourceConfig::Standard(FlattenedPair {
+                first: StandardSource {
+                    address: NormalizedAddress::new_unchecked("example.com", 123).into(),
+                    ntp_version: ProtocolVersion::v4_upgrading_to_v5_with_default_tries(),
+                },
+                second: Default::default()
+            })]
+        );
+        assert!(config.observability.log_level.is_none());
     }
 
     #[test]
