@@ -290,7 +290,10 @@ where
                             .ok();
                     }
                     ntp_proto::NtpSourceAction::SetTimer(timeout) => {
-                        poll_wait.as_mut().reset(Instant::now() + timeout)
+                        if let Some(deadline) = Instant::now().checked_add(timeout) {
+                            // If it overflows, it is so far in the future we may as well not set the timer.
+                            poll_wait.as_mut().reset(deadline);
+                        }
                     }
                     ntp_proto::NtpSourceAction::Reset => {
                         self.channels
@@ -452,7 +455,7 @@ mod tests {
 
     use ntp_proto::{
         AlgorithmConfig, KalmanClockController, KalmanControllerMessage, KalmanSourceMessage,
-        NoCipher, NtpDuration, NtpLeapIndicator, NtpPacket, ProtocolVersion, SourceDefaultsConfig,
+        NoCipher, NtpDuration, NtpLeapIndicator, NtpPacket, ProtocolVersion, SourceConfig,
         SynchronizationConfig, SystemSnapshot, TimeSnapshot, TwoWayKalmanSourceController,
     };
     use timestamped_socket::socket::{open_ip, GeneralTimestampMode, Open};
@@ -599,7 +602,6 @@ mod tests {
         let mut system: ntp_proto::System<_, KalmanClockController<_, _>> = ntp_proto::System::new(
             TestClock {},
             SynchronizationConfig::default(),
-            SourceDefaultsConfig::default(),
             AlgorithmConfig::default(),
             Arc::new([]),
         )
@@ -607,8 +609,9 @@ mod tests {
 
         let Ok((source, _)) = system.create_ntp_source(
             index,
+            SourceConfig::default(),
             SocketAddr::from((Ipv4Addr::LOCALHOST, port_base)),
-            ProtocolVersion::default(),
+            ProtocolVersion::V4,
             None,
         ) else {
             panic!("Could not create test source");
@@ -732,31 +735,39 @@ mod tests {
             process.run(poll_wait).await;
         });
 
+        for _ in 0..3 {
+            poll_send.notify();
+
+            let mut buf = [0; 48];
+            let RecvResult {
+                bytes_read: size,
+                timestamp,
+                remote_addr,
+            } = socket.recv(&mut buf).await.unwrap();
+            assert_eq!(size, 48);
+            assert!(timestamp.is_some());
+
+            let rec_packet = NtpPacket::deserialize(&buf, &NoCipher).unwrap().0;
+            let send_packet = NtpPacket::deny_response(rec_packet);
+            let serialized = serialize_packet_unencrypted(&send_packet);
+
+            // Flush earlier messages
+            while msg_recv.try_recv().is_ok() {}
+
+            socket
+                .send_to(&serialized, std::dbg!(remote_addr))
+                .await
+                .unwrap();
+
+            tokio::time::sleep(Duration::from_millis(10)).await;
+        }
+
         poll_send.notify();
 
-        let mut buf = [0; 48];
-        let RecvResult {
-            bytes_read: size,
-            timestamp,
-            remote_addr,
-        } = socket.recv(&mut buf).await.unwrap();
-        assert_eq!(size, 48);
-        assert!(timestamp.is_some());
-
-        let rec_packet = NtpPacket::deserialize(&buf, &NoCipher).unwrap().0;
-        let send_packet = NtpPacket::deny_response(rec_packet);
-        let serialized = serialize_packet_unencrypted(&send_packet);
-
-        // Flush earlier messages
-        while msg_recv.try_recv().is_ok() {}
-
-        socket.send_to(&serialized, remote_addr).await.unwrap();
-
-        let msg = msg_recv.recv().await.unwrap();
+        let msg = dbg!(msg_recv.recv().await.unwrap());
         assert!(matches!(msg, MsgForSystem::MustDemobilize(_)));
 
-        poll_send.notify();
-
+        let mut buf = [0; 48];
         tokio::select! {
             _ = tokio::time::sleep(Duration::from_millis(10)) => {/*expected */},
             _ = socket.recv(&mut buf) => { unreachable!("should not receive anything") }
