@@ -1,9 +1,7 @@
 use std::fmt::Display;
 use std::{net::SocketAddr, ops::Deref};
 
-#[cfg(feature = "unstable_ntpv5")]
-use ntp_proto::NtpVersion;
-use ntp_proto::ProtocolVersion;
+use ntp_proto::SourceConfig;
 use tokio::sync::mpsc;
 use tracing::warn;
 
@@ -18,6 +16,7 @@ struct PoolSource {
 
 pub struct PoolSpawner {
     config: PoolSourceConfig,
+    source_config: SourceConfig,
     id: SpawnerId,
     current_sources: Vec<PoolSource>,
     known_ips: Vec<SocketAddr>,
@@ -35,9 +34,10 @@ impl Display for PoolSpawnError {
 impl std::error::Error for PoolSpawnError {}
 
 impl PoolSpawner {
-    pub fn new(config: PoolSourceConfig) -> PoolSpawner {
+    pub fn new(config: PoolSourceConfig, source_config: SourceConfig) -> PoolSpawner {
         PoolSpawner {
             config,
+            source_config,
             id: Default::default(),
             current_sources: Default::default(),
             known_ips: Default::default(),
@@ -85,14 +85,8 @@ impl Spawner for PoolSpawner {
                     id,
                     addr,
                     self.config.addr.deref().clone(),
-                    #[cfg(not(feature = "unstable_ntpv5"))]
-                    ProtocolVersion::default(),
-                    #[cfg(feature = "unstable_ntpv5")]
-                    match self.config.ntp_version {
-                        Some(NtpVersion::V4) => ProtocolVersion::V4,
-                        Some(NtpVersion::V5) => ProtocolVersion::V5,
-                        None => ProtocolVersion::default(),
-                    },
+                    self.config.ntp_version,
+                    self.source_config,
                     None,
                 );
                 tracing::debug!(?action, "intending to spawn new pool source at");
@@ -136,9 +130,9 @@ impl Spawner for PoolSpawner {
 
 #[cfg(test)]
 mod tests {
-    #[cfg(feature = "unstable_ntpv5")]
     use ntp_proto::ProtocolVersion;
 
+    use ntp_proto::SourceConfig;
     use tokio::sync::mpsc::{self, error::TryRecvError};
 
     use crate::daemon::{
@@ -155,14 +149,16 @@ mod tests {
         let address_strings = ["127.0.0.1:123", "127.0.0.2:123", "127.0.0.3:123"];
         let addresses = address_strings.map(|addr| addr.parse().unwrap());
 
-        let mut pool = PoolSpawner::new(PoolSourceConfig {
-            addr: NormalizedAddress::with_hardcoded_dns("example.com", 123, addresses.to_vec())
-                .into(),
-            count: 2,
-            ignore: vec![],
-            #[cfg(feature = "unstable_ntpv5")]
-            ntp_version: None,
-        });
+        let mut pool = PoolSpawner::new(
+            PoolSourceConfig {
+                addr: NormalizedAddress::with_hardcoded_dns("example.com", 123, addresses.to_vec())
+                    .into(),
+                count: 2,
+                ignore: vec![],
+                ntp_version: ProtocolVersion::v4_upgrading_to_v5_with_default_tries(),
+            },
+            SourceConfig::default(),
+        );
         let spawner_id = pool.get_id();
         let (action_tx, mut action_rx) = mpsc::channel(MESSAGE_BUFFER_SIZE);
 
@@ -172,20 +168,18 @@ mod tests {
         assert_eq!(spawner_id, res.id);
         let params = get_ntp_create_params(res).unwrap();
         let addr1 = params.addr;
-        #[cfg(feature = "unstable_ntpv5")]
         assert_eq!(
             params.protocol_version,
-            ProtocolVersion::V4UpgradingToV5 { tries_left: 8 }
+            ProtocolVersion::v4_upgrading_to_v5_with_default_tries()
         );
 
         let res = action_rx.try_recv().unwrap();
         assert_eq!(spawner_id, res.id);
         let params = get_ntp_create_params(res).unwrap();
         let addr2 = params.addr;
-        #[cfg(feature = "unstable_ntpv5")]
         assert_eq!(
             params.protocol_version,
-            ProtocolVersion::V4UpgradingToV5 { tries_left: 8 }
+            ProtocolVersion::v4_upgrading_to_v5_with_default_tries(),
         );
 
         assert_ne!(addr1, addr2);
@@ -197,20 +191,21 @@ mod tests {
         assert!(pool.is_complete());
     }
 
-    #[cfg(feature = "unstable_ntpv5")]
     #[tokio::test]
     async fn respects_ntp_version_force_v5() {
         let address_strings = ["127.0.0.1:123", "127.0.0.2:123", "127.0.0.3:123"];
         let addresses = address_strings.map(|addr| addr.parse().unwrap());
 
-        let mut pool = PoolSpawner::new(PoolSourceConfig {
-            addr: NormalizedAddress::with_hardcoded_dns("example.com", 123, addresses.to_vec())
-                .into(),
-            count: 2,
-            ignore: vec![],
-            #[cfg(feature = "unstable_ntpv5")]
-            ntp_version: Some(ntp_proto::NtpVersion::V5),
-        });
+        let mut pool = PoolSpawner::new(
+            PoolSourceConfig {
+                addr: NormalizedAddress::with_hardcoded_dns("example.com", 123, addresses.to_vec())
+                    .into(),
+                count: 2,
+                ignore: vec![],
+                ntp_version: ProtocolVersion::V5,
+            },
+            SourceConfig::default(),
+        );
         let spawner_id = pool.get_id();
         let (action_tx, mut action_rx) = mpsc::channel(MESSAGE_BUFFER_SIZE);
 
@@ -220,14 +215,12 @@ mod tests {
         assert_eq!(spawner_id, res.id);
         let params = get_ntp_create_params(res).unwrap();
         let addr1 = params.addr;
-        #[cfg(feature = "unstable_ntpv5")]
         assert_eq!(params.protocol_version, ProtocolVersion::V5);
 
         let res = action_rx.try_recv().unwrap();
         assert_eq!(spawner_id, res.id);
         let params = get_ntp_create_params(res).unwrap();
         let addr2 = params.addr;
-        #[cfg(feature = "unstable_ntpv5")]
         assert_eq!(params.protocol_version, ProtocolVersion::V5);
 
         assert_ne!(addr1, addr2);
@@ -239,20 +232,21 @@ mod tests {
         assert!(pool.is_complete());
     }
 
-    #[cfg(feature = "unstable_ntpv5")]
     #[tokio::test]
     async fn respects_ntp_version_force_v4() {
         let address_strings = ["127.0.0.1:123", "127.0.0.2:123", "127.0.0.3:123"];
         let addresses = address_strings.map(|addr| addr.parse().unwrap());
 
-        let mut pool = PoolSpawner::new(PoolSourceConfig {
-            addr: NormalizedAddress::with_hardcoded_dns("example.com", 123, addresses.to_vec())
-                .into(),
-            count: 2,
-            ignore: vec![],
-            #[cfg(feature = "unstable_ntpv5")]
-            ntp_version: Some(ntp_proto::NtpVersion::V4),
-        });
+        let mut pool = PoolSpawner::new(
+            PoolSourceConfig {
+                addr: NormalizedAddress::with_hardcoded_dns("example.com", 123, addresses.to_vec())
+                    .into(),
+                count: 2,
+                ignore: vec![],
+                ntp_version: ProtocolVersion::V4,
+            },
+            SourceConfig::default(),
+        );
         let spawner_id = pool.get_id();
         let (action_tx, mut action_rx) = mpsc::channel(MESSAGE_BUFFER_SIZE);
 
@@ -262,14 +256,12 @@ mod tests {
         assert_eq!(spawner_id, res.id);
         let params = get_ntp_create_params(res).unwrap();
         let addr1 = params.addr;
-        #[cfg(feature = "unstable_ntpv5")]
         assert_eq!(params.protocol_version, ProtocolVersion::V4);
 
         let res = action_rx.try_recv().unwrap();
         assert_eq!(spawner_id, res.id);
         let params = get_ntp_create_params(res).unwrap();
         let addr2 = params.addr;
-        #[cfg(feature = "unstable_ntpv5")]
         assert_eq!(params.protocol_version, ProtocolVersion::V4);
 
         assert_ne!(addr1, addr2);
@@ -287,14 +279,16 @@ mod tests {
         let addresses = address_strings.map(|addr| addr.parse().unwrap());
         let ignores = vec!["127.0.0.1".parse().unwrap()];
 
-        let mut pool = PoolSpawner::new(PoolSourceConfig {
-            addr: NormalizedAddress::with_hardcoded_dns("example.com", 123, addresses.to_vec())
-                .into(),
-            count: 2,
-            ignore: ignores.clone(),
-            #[cfg(feature = "unstable_ntpv5")]
-            ntp_version: None,
-        });
+        let mut pool = PoolSpawner::new(
+            PoolSourceConfig {
+                addr: NormalizedAddress::with_hardcoded_dns("example.com", 123, addresses.to_vec())
+                    .into(),
+                count: 2,
+                ignore: ignores.clone(),
+                ntp_version: ProtocolVersion::v4_upgrading_to_v5_with_default_tries(),
+            },
+            SourceConfig::default(),
+        );
         let spawner_id = pool.get_id();
         let (action_tx, mut action_rx) = mpsc::channel(MESSAGE_BUFFER_SIZE);
 
@@ -326,14 +320,16 @@ mod tests {
         let address_strings = ["127.0.0.1:123", "127.0.0.2:123", "127.0.0.3:123"];
         let addresses = address_strings.map(|addr| addr.parse().unwrap());
 
-        let mut pool = PoolSpawner::new(PoolSourceConfig {
-            addr: NormalizedAddress::with_hardcoded_dns("example.com", 123, addresses.to_vec())
-                .into(),
-            count: 2,
-            ignore: vec![],
-            #[cfg(feature = "unstable_ntpv5")]
-            ntp_version: None,
-        });
+        let mut pool = PoolSpawner::new(
+            PoolSourceConfig {
+                addr: NormalizedAddress::with_hardcoded_dns("example.com", 123, addresses.to_vec())
+                    .into(),
+                count: 2,
+                ignore: vec![],
+                ntp_version: ProtocolVersion::v4_upgrading_to_v5_with_default_tries(),
+            },
+            SourceConfig::default(),
+        );
         let (action_tx, mut action_rx) = mpsc::channel(MESSAGE_BUFFER_SIZE);
 
         assert!(!pool.is_complete());
@@ -370,13 +366,15 @@ mod tests {
 
     #[tokio::test]
     async fn works_if_address_does_not_resolve() {
-        let mut pool = PoolSpawner::new(PoolSourceConfig {
-            addr: NormalizedAddress::with_hardcoded_dns("does.not.resolve", 123, vec![]).into(),
-            count: 2,
-            ignore: vec![],
-            #[cfg(feature = "unstable_ntpv5")]
-            ntp_version: None,
-        });
+        let mut pool = PoolSpawner::new(
+            PoolSourceConfig {
+                addr: NormalizedAddress::with_hardcoded_dns("does.not.resolve", 123, vec![]).into(),
+                count: 2,
+                ignore: vec![],
+                ntp_version: ProtocolVersion::v4_upgrading_to_v5_with_default_tries(),
+            },
+            SourceConfig::default(),
+        );
         let (action_tx, mut action_rx) = mpsc::channel(MESSAGE_BUFFER_SIZE);
         assert!(!pool.is_complete());
         pool.try_spawn(&action_tx).await.unwrap();
