@@ -1,9 +1,7 @@
 use std::fmt::Display;
 use std::{net::SocketAddr, ops::Deref};
 
-#[cfg(feature = "unstable_ntpv5")]
-use ntp_proto::NtpVersion;
-use ntp_proto::ProtocolVersion;
+use ntp_proto::SourceConfig;
 use tokio::sync::mpsc;
 use tracing::warn;
 
@@ -16,6 +14,7 @@ use super::{
 pub struct StandardSpawner {
     id: SpawnerId,
     config: StandardSource,
+    source_config: SourceConfig,
     resolved: Option<SocketAddr>,
     has_spawned: bool,
 }
@@ -42,10 +41,11 @@ impl From<mpsc::error::SendError<SpawnEvent>> for StandardSpawnError {
 impl std::error::Error for StandardSpawnError {}
 
 impl StandardSpawner {
-    pub fn new(config: StandardSource) -> StandardSpawner {
+    pub fn new(config: StandardSource, source_config: SourceConfig) -> StandardSpawner {
         StandardSpawner {
             id: Default::default(),
             config,
+            source_config,
             resolved: None,
             has_spawned: false,
         }
@@ -93,14 +93,8 @@ impl Spawner for StandardSpawner {
                     SourceId::new(),
                     addr,
                     self.config.address.deref().clone(),
-                    #[cfg(not(feature = "unstable_ntpv5"))]
-                    ProtocolVersion::default(),
-                    #[cfg(feature = "unstable_ntpv5")]
-                    match self.config.ntp_version {
-                        Some(NtpVersion::V4) => ProtocolVersion::V4,
-                        Some(NtpVersion::V5) => ProtocolVersion::V5,
-                        None => ProtocolVersion::default(),
-                    },
+                    self.config.ntp_version,
+                    self.source_config,
                     None,
                 ),
             ))
@@ -142,9 +136,9 @@ impl Spawner for StandardSpawner {
 
 #[cfg(test)]
 mod tests {
-    #[cfg(feature = "unstable_ntpv5")]
     use ntp_proto::ProtocolVersion;
 
+    use ntp_proto::SourceConfig;
     use tokio::sync::mpsc::{self, error::TryRecvError};
 
     use crate::daemon::{
@@ -158,16 +152,18 @@ mod tests {
 
     #[tokio::test]
     async fn creates_a_source() {
-        let mut spawner = StandardSpawner::new(StandardSource {
-            address: NormalizedAddress::with_hardcoded_dns(
-                "example.com",
-                123,
-                vec!["127.0.0.1:123".parse().unwrap()],
-            )
-            .into(),
-            #[cfg(feature = "unstable_ntpv5")]
-            ntp_version: None,
-        });
+        let mut spawner = StandardSpawner::new(
+            StandardSource {
+                address: NormalizedAddress::with_hardcoded_dns(
+                    "example.com",
+                    123,
+                    vec!["127.0.0.1:123".parse().unwrap()],
+                )
+                .into(),
+                ntp_version: ProtocolVersion::v4_upgrading_to_v5_with_default_tries(),
+            },
+            SourceConfig::default(),
+        );
         let spawner_id = spawner.get_id();
         let (action_tx, mut action_rx) = mpsc::channel(MESSAGE_BUFFER_SIZE);
 
@@ -179,28 +175,29 @@ mod tests {
         assert_eq!(create_params.get_addr(), "127.0.0.1:123");
         let params = get_ntp_create_params(res).unwrap();
         assert_eq!(params.addr.to_string(), "127.0.0.1:123");
-        #[cfg(feature = "unstable_ntpv5")]
         assert_eq!(
             params.protocol_version,
-            ProtocolVersion::V4UpgradingToV5 { tries_left: 8 }
+            ProtocolVersion::v4_upgrading_to_v5_with_default_tries()
         );
 
         // Should be complete after spawning
         assert!(spawner.is_complete());
     }
 
-    #[cfg(feature = "unstable_ntpv5")]
     #[tokio::test]
     async fn respects_ntp_version_force_v5() {
-        let mut spawner = StandardSpawner::new(StandardSource {
-            address: NormalizedAddress::with_hardcoded_dns(
-                "example.com",
-                123,
-                vec!["127.0.0.1:123".parse().unwrap()],
-            )
-            .into(),
-            ntp_version: Some(ntp_proto::NtpVersion::V5),
-        });
+        let mut spawner = StandardSpawner::new(
+            StandardSource {
+                address: NormalizedAddress::with_hardcoded_dns(
+                    "example.com",
+                    123,
+                    vec!["127.0.0.1:123".parse().unwrap()],
+                )
+                .into(),
+                ntp_version: ProtocolVersion::V5,
+            },
+            SourceConfig::default(),
+        );
         let spawner_id = spawner.get_id();
         let (action_tx, mut action_rx) = mpsc::channel(MESSAGE_BUFFER_SIZE);
 
@@ -216,18 +213,20 @@ mod tests {
         assert!(spawner.is_complete());
     }
 
-    #[cfg(feature = "unstable_ntpv5")]
     #[tokio::test]
     async fn respects_ntp_version_force_v4() {
-        let mut spawner = StandardSpawner::new(StandardSource {
-            address: NormalizedAddress::with_hardcoded_dns(
-                "example.com",
-                123,
-                vec!["127.0.0.1:123".parse().unwrap()],
-            )
-            .into(),
-            ntp_version: Some(ntp_proto::NtpVersion::V4),
-        });
+        let mut spawner = StandardSpawner::new(
+            StandardSource {
+                address: NormalizedAddress::with_hardcoded_dns(
+                    "example.com",
+                    123,
+                    vec!["127.0.0.1:123".parse().unwrap()],
+                )
+                .into(),
+                ntp_version: ProtocolVersion::V4,
+            },
+            SourceConfig::default(),
+        );
         let spawner_id = spawner.get_id();
         let (action_tx, mut action_rx) = mpsc::channel(MESSAGE_BUFFER_SIZE);
 
@@ -245,16 +244,18 @@ mod tests {
 
     #[tokio::test]
     async fn recreates_a_source() {
-        let mut spawner = StandardSpawner::new(StandardSource {
-            address: NormalizedAddress::with_hardcoded_dns(
-                "example.com",
-                123,
-                vec!["127.0.0.1:123".parse().unwrap()],
-            )
-            .into(),
-            #[cfg(feature = "unstable_ntpv5")]
-            ntp_version: None,
-        });
+        let mut spawner = StandardSpawner::new(
+            StandardSource {
+                address: NormalizedAddress::with_hardcoded_dns(
+                    "example.com",
+                    123,
+                    vec!["127.0.0.1:123".parse().unwrap()],
+                )
+                .into(),
+                ntp_version: ProtocolVersion::v4_upgrading_to_v5_with_default_tries(),
+            },
+            SourceConfig::default(),
+        );
         let (action_tx, mut action_rx) = mpsc::channel(MESSAGE_BUFFER_SIZE);
 
         assert!(!spawner.is_complete());
@@ -284,16 +285,18 @@ mod tests {
         let address_strings = ["127.0.0.1:123", "127.0.0.2:123", "127.0.0.3:123"];
         let addresses = address_strings.map(|addr| addr.parse().unwrap());
 
-        let mut spawner = StandardSpawner::new(StandardSource {
-            address: NormalizedAddress::with_hardcoded_dns(
-                "europe.pool.ntp.org",
-                123,
-                addresses.to_vec(),
-            )
-            .into(),
-            #[cfg(feature = "unstable_ntpv5")]
-            ntp_version: None,
-        });
+        let mut spawner = StandardSpawner::new(
+            StandardSource {
+                address: NormalizedAddress::with_hardcoded_dns(
+                    "europe.pool.ntp.org",
+                    123,
+                    addresses.to_vec(),
+                )
+                .into(),
+                ntp_version: ProtocolVersion::v4_upgrading_to_v5_with_default_tries(),
+            },
+            SourceConfig::default(),
+        );
         let (action_tx, mut action_rx) = mpsc::channel(MESSAGE_BUFFER_SIZE);
 
         assert!(!spawner.is_complete());
@@ -327,27 +330,26 @@ mod tests {
         for addr in seen_addresses.iter() {
             assert!(
                 addresses.contains(addr),
-                "{:?} should have been drawn from {:?}",
-                addr,
-                addresses
+                "{addr:?} should have been drawn from {addresses:?}"
             );
         }
 
         assert!(
             seen_addresses.iter().any(|seen| seen != &initial_addr),
-            "Re-resolved\n\n\t{:?}\n\n should contain at least one address that isn't the original\n\n\t{:?}",
-            seen_addresses,
-            initial_addr,
+            "Re-resolved\n\n\t{seen_addresses:?}\n\n should contain at least one address that isn't the original\n\n\t{initial_addr:?}",
         );
     }
 
     #[tokio::test]
     async fn works_if_address_does_not_resolve() {
-        let mut spawner = StandardSpawner::new(StandardSource {
-            address: NormalizedAddress::with_hardcoded_dns("does.not.resolve", 123, vec![]).into(),
-            #[cfg(feature = "unstable_ntpv5")]
-            ntp_version: None,
-        });
+        let mut spawner = StandardSpawner::new(
+            StandardSource {
+                address: NormalizedAddress::with_hardcoded_dns("does.not.resolve", 123, vec![])
+                    .into(),
+                ntp_version: ProtocolVersion::v4_upgrading_to_v5_with_default_tries(),
+            },
+            SourceConfig::default(),
+        );
         let (action_tx, mut action_rx) = mpsc::channel(MESSAGE_BUFFER_SIZE);
 
         spawner.try_spawn(&action_tx).await.unwrap();
