@@ -6,7 +6,7 @@
 # Because this script generate keys without passwords set, they should only be used in a development setting.
 
 if [ -z "$1" ]; then
-	echo "usage: gen-cert.sh name-of-server [ca-name]"
+	echo "usage: gen-cert.sh name-of-server [ca-name] [filename]"
 	echo
 	echo "This will generate a name-of-server.key, name-of-server.pem and name-of-server.chain.pem file"
 	echo "containing the private key, public certificate, and full certificate chain (respectively)"
@@ -18,18 +18,20 @@ fi
 
 NAME="${1:-ntpd-rs.test}"
 CA="${2:-testca}"
+FILENAME="${3:-$NAME}"
 
 # generate a key
-openssl genrsa -out "$NAME".key 2048
+openssl genrsa -out "$FILENAME".key 2048
 
 # generate a certificate signing request
-openssl req -batch -new -key "$NAME".key -out "$NAME".csr
+openssl req -batch -new -key "$FILENAME".key -out "$FILENAME".csr
 
 # generate an ext file
-cat >> "$NAME".ext <<EOF
+cat >> "$FILENAME".ext <<EOF
 authorityKeyIdentifier=keyid,issuer
 basicConstraints=CA:FALSE
-keyUsage = keyEncipherment
+keyUsage = digitalSignature, keyEncipherment, keyAgreement
+extendedKeyUsage = serverAuth
 subjectAltName = @alt_names
 
 [alt_names]
@@ -37,10 +39,10 @@ DNS.1 = $NAME
 EOF
 
 # generate the signed certificate with the provided CA
-openssl x509 -req -in "$NAME".csr -CA "$CA".pem -CAkey "$CA".key -CAcreateserial -out "$NAME".pem -days 1825 -sha256 -extfile "$NAME".ext
+openssl x509 -req -in "$FILENAME".csr -CA "$CA".pem -CAkey "$CA".key -CAcreateserial -out "$FILENAME".pem -days 365 -sha256 -extfile "$FILENAME".ext
 
 # generate the full certificate chain version
-cat "$NAME".pem "$CA".pem > "$NAME".chain.pem
+cat "$FILENAME".pem "$CA".pem > "$FILENAME".fullchain.pem
 
 # cleanup
-rm "$NAME".csr
+rm "$FILENAME".csr
